import * as effector from 'effector'
import { Unit } from 'effector'
export { ConfigPersist, Done, Fail, Finally, Persist, StorageAdapter } from '..'

declare type Done<State> = {
  key: string
  keyPrefix: string
  operation: 'set' | 'get'
  value: State
}
declare type Fail<Err> = {
  key: string
  keyPrefix: string
  operation: 'set' | 'get'
  error: Err
  value?: any
}
declare type Finally<State, Err> =
  | (Done<State> & {
      status: 'done'
    })
  | (Fail<Err> & {
      status: 'fail'
    })
interface ConfigCommon<State, Err = Error> {
  clock?: Unit<any>
  done?: Unit<Done<State>>
  fail?: Unit<Fail<Err>>
  finally?: Unit<Finally<State, Err>>
  pickup?: Unit<any>
  key?: string
  keyPrefix?: string
}

interface ConfigStore<State, Err = Error> extends ConfigCommon<State, Err> {}
/**
 * Partially applied `persist` with predefined `memory` adapter and curried `store`
 */
declare const persist: (
  config?: ConfigStore<any, Error> | undefined
) => <State>(store: effector.Store<State>) => effector.Store<State>

export { ConfigStore, persist }
